﻿using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;

namespace TestGridViewEditScroll
{
	public partial class TestGridViewEditScrollViewController : UIViewController
	{
		#region Members
		IGGridView gridView;

		IGGridViewDataSourceHelper dsh;

		List<cSimpleGrid> lstGridItems;

		#endregion

		#region Constructor

		public TestGridViewEditScrollViewController () : base ("TestGridViewEditScrollViewController", null)
		{
		}

		#endregion

		#region View Controller Methods

		public override void DidReceiveMemoryWarning ()
		{
			// Releases the view if it doesn't have a superview.
			base.DidReceiveMemoryWarning ();
			
			// Release any cached data, images, etc that aren't in use.
		}

		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();
			
			//Next, create a new instance of the grid view and place it on the view.
			gridView = new IGGridView (new System.Drawing.RectangleF (0, 0, this.View.Bounds.Width, this.View.Bounds.Height), IGGridViewStyle.IGGridViewStyleDefault);
			gridView.AutoresizingMask = UIViewAutoresizing.All;
			gridView.HeaderHeight = 65f;
			gridView.RowReordering = true;

			lstGridItems = new List<cSimpleGrid> ();

			cSimpleGrid gridItem = new cSimpleGrid ();

			gridItem ["Test"] = "Test 1";
			lstGridItems.Add (gridItem);

			gridItem = new cSimpleGrid ();
			gridItem ["Test"] = "Test 2";
			lstGridItems.Add (gridItem);

			gridItem = new cSimpleGrid ();
			gridItem ["Test"] = "Test 3";
			lstGridItems.Add (gridItem);

			gridItem = new cSimpleGrid ();
			gridItem ["Test"] = "Test 4";
			lstGridItems.Add (gridItem);

			gridItem = new cSimpleGrid ();
			gridItem ["Test"] = "Test 5";
			lstGridItems.Add (gridItem);


			this.View.Add (gridView);

			SetupGrid ();
		}

		#endregion

		/// <summary>
		/// This method is responsible for the layour and populate of the grid view
		/// </summary>
		private bool SetupGrid ()
		{
			//First, create a new return variable
			bool bReturn = true;

			try {

				// We're using the basic IGGridViewDataSourceHelper
				// since our data will be displayed in a standard datagrid
				dsh = new IGGridViewDataSourceHelper();

				dsh.EditingDelegate = new customGridViewDataSourceEditingDelegate();

				// Since we are manually creating columns, turn off auto-generation
				dsh.AutoGenerateColumns = false;

			
				var column = new Dictionary<String, customSimpleGridColumnDefinition> ();
				column ["Test"] = new customSimpleGridColumnDefinition ("Test");
				dsh.ColumnDefinitions.Add (column ["Test"]);

				//Next, set the data object
				dsh.Data = lstGridItems.ToArray ();

				//Allow editing of the grid View
				dsh.AllowEditing = true;

				//Set the DataSource
				gridView.DataSource = dsh;

				gridView.RowReordering = true;

				//Turn off empty rows
				gridView.EmptyRows = false;

				//Update gridView
				gridView.UpdateData ();

			} catch {
				//Error building the grid, return false.
				bReturn = false;
			}

			//Finally, return the return variable
			return bReturn;
		}


		#region Specialized Classes

		public class customSimpleGridColumnDefinition : IGGridViewColumnDefinition
		{
			string _fieldKey;

			public customSimpleGridColumnDefinition (string key) : base ()
			{
				_fieldKey = key;
			}

			public override string FieldKey {
				get {
					return _fieldKey;
				}
			}


			public override bool Editiable {
				get {
					return true;
				}
				set {
					base.Editiable = true;
				}
			}

			public override IGGridViewCell CreateCell (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
			{
				IGGridViewCell cell = (IGGridViewCell)gridView.DequeueReusableCell ("simpleGridCell");

				if (cell == null) {
					cell = new IGGridViewCell ("simpleGridCell");
				}

				cSimpleGrid dataRow = (cSimpleGrid)dataSource.ResolveDataObjectForRow (path);

				cell.TextLabel.Text = dataRow [_fieldKey].ToString ();

				//Before returning the cell, apply the styling to the cell
				ApplyStylingToCell (cell, path.RowIndex, gridView);

				return cell;
			}

			public override void RemoveEditor (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource, bool cellVisible, bool anotherCellEnteringEditMode)
			{
				gridView.UnregisterKeyboard (!anotherCellEnteringEditMode);

				IGGridViewCell cell = (IGGridViewCell)gridView.ResolveCellForPath (path);

				UITextField field = (UITextField)cell.Subviews [0];

				cSimpleGrid dataRow = (cSimpleGrid)dataSource.ResolveDataObjectForRow (path);

				dataRow [_fieldKey] = cell.TextLabel.Text =  field.Text;

				float duration = cellVisible? dataSource.EditingTransitionDuration : 0.0f;

				UIView.Transition (field, cell.TextLabel, duration, UIViewAnimationOptions.TransitionFlipFromTop, null);

			}


		}

		public class customGridViewDataSourceEditingDelegate : IGGridViewDataSourceHelperEditingDelegate
		{
			#region Members

			Keyboard keyboard;

			Accessory accessory;

			#endregion

			#region Constructor

			public customGridViewDataSourceEditingDelegate()
			{

			}

			#endregion

			#region Override Methods

			public override void CellEnteringEditMode (IGGridViewDataSourceHelper dsh, IGGridViewCell cell, NSObject val, UIView editor, IGGridViewColumnDefinition col)
			{
				keyboard = new Keyboard ();
				accessory = new Accessory((UITextField)editor);

				((UITextField)editor).InputView = keyboard.View;
				((UITextField)editor).InputAccessoryView = accessory.View;

			}

			#endregion
		}

		#endregion

	}
}

